//------------------------------------------------------------------------------
//! @file KnownSystemName.h
//! @brief Generated KnownSystemName enum
//
// SPDX-FileCopyrightText: Michael Popoloski
// SPDX-License-Identifier: MIT
//------------------------------------------------------------------------------
#pragma once

#include <array>
#include <ostream>

#include "slang/util/Util.h"

namespace slang::parsing {

enum class SLANG_EXPORT KnownSystemName {
    Unknown,
    Bits,
    Typename,
    IsUnbounded,
    Low,
    High,
    Left,
    Right,
    Size,
    Increment,
    Dimensions,
    UnpackedDimensions,
    Rtoi,
    Itor,
    RealToBits,
    BitsToReal,
    ShortrealToBits,
    BitsToShortreal,
    Signed,
    Unsigned,
    CoverageControl,
    CoverageGetMax,
    CoverageGet,
    CoverageMerge,
    CoverageSave,
    GetCoverage,
    SetCoverageDbName,
    LoadCoverageDb,
    Clog2,
    CountBits,
    CountOnes,
    OneHot,
    OneHot0,
    IsUnknown,
    Ln,
    Log10,
    Exp,
    Sqrt,
    Floor,
    Ceil,
    Sin,
    Cos,
    Tan,
    Asin,
    Acos,
    Atan,
    Sinh,
    Cosh,
    Tanh,
    Asinh,
    Acosh,
    Atanh,
    Pow,
    Atan2,
    Hypot,
    ValuePlusArgs,
    GlobalClock,
    SFormatF,
    PSPrintF,
    InferredClock,
    InferredDisable,
    Randomize,
    Triggered,
    Matched,
    FError,
    FGets,
    FScanf,
    SScanf,
    FRead,
    FOpen,
    FClose,
    FGetC,
    UngetC,
    FTell,
    FSeek,
    Rewind,
    FFlush,
    FEof,
    Rose,
    Fell,
    Stable,
    Changed,
    PastGclk,
    RoseGclk,
    FellGclk,
    StableGclk,
    ChangedGclk,
    FutureGclk,
    RisingGclk,
    FallingGclk,
    SteadyGclk,
    ChangingGclk,
    Sampled,
    Past,
    Stacktrace,
    CountDrivers,
    GetPattern,
    TestPlusArgs,
    Time,
    STime,
    RealTime,
    Random,
    URandom,
    URandomRange,
    ResetCount,
    ResetValue,
    TimeUnit,
    TimePrecision,
    Scale,
    SdfAnnotate,
    DistUniform,
    DistNormal,
    DistExponential,
    DistPoisson,
    DistChiSquare,
    DistT,
    DistErlang,
    Fatal,
    StaticAssert,
    Cast,
    Info,
    Warning,
    Error,
    Finish,
    Stop,
    AssertControl,
    AssertOn,
    AssertOff,
    AssertKill,
    AssertPassOn,
    AssertPassOff,
    AssertFailOn,
    AssertFailOff,
    AssertNonvacuousOn,
    AssertVacuousOff,
    Display,
    DisplayB,
    DisplayO,
    DisplayH,
    Write,
    WriteB,
    WriteO,
    WriteH,
    Strobe,
    StrobeB,
    StrobeO,
    StrobeH,
    Monitor,
    MonitorB,
    MonitorO,
    MonitorH,
    FDisplay,
    FDisplayB,
    FDisplayO,
    FDisplayH,
    FWrite,
    FWriteB,
    FWriteO,
    FWriteH,
    FStrobe,
    FStrobeB,
    FStrobeO,
    FStrobeH,
    FMonitor,
    FMonitorB,
    FMonitorO,
    FMonitorH,
    SWrite,
    SWriteB,
    SWriteO,
    SWriteH,
    SFormat,
    PrintTimeScale,
    DumpVars,
    DumpPorts,
    ShowVars,
    ReadMemB,
    ReadMemH,
    WriteMemB,
    WriteMemH,
    SReadMemB,
    SReadMemH,
    System,
    List,
    Scope,
    Exit,
    TimeFormat,
    MonitorOn,
    MonitorOff,
    DumpFile,
    DumpOn,
    DumpOff,
    DumpAll,
    DumpLimit,
    DumpFlush,
    DumpPortsOn,
    DumpPortsOff,
    DumpPortsAll,
    DumpPortsLimit,
    DumpPortsFlush,
    Input,
    Key,
    NoKey,
    Log,
    NoLog,
    Reset,
    Save,
    Restart,
    IncSave,
    ShowScopes,
    QInitialize,
    QAdd,
    QRemove,
    QExam,
    QFull,
    AsyncAndArray,
    SyncAndArray,
    AsyncAndPlane,
    SyncAndPlane,
    AsyncNandArray,
    SyncNandArray,
    AsyncNandPlane,
    SyncNandPlane,
    AsyncOrArray,
    SyncOrArray,
    AsyncOrPlane,
    SyncOrPlane,
    AsyncNorArray,
    SyncNorArray,
    AsyncNorPlane,
    SyncNorPlane,
    Reverse,
    Delete,
    Exists,
    Insert,
    Index,
    Map,
    ArraySize,
    Or,
    And,
    XOr,
    Sum,
    Product,
    Find,
    FindIndex,
    FindFirst,
    FindFirstIndex,
    FindLast,
    FindLastIndex,
    Min,
    Max,
    Unique,
    UniqueIndex,
    Sort,
    Rsort,
    Shuffle,
    Num,
    First,
    Last,
    Next,
    Prev,
    PopFront,
    PopBack,
    PushFront,
    PushBack,
    Name,
    Len,
    Putc,
    Getc,
    Substr,
    ToUpper,
    ToLower,
    Compare,
    ICompare,
    AToI,
    AToHex,
    AToOct,
    AToBin,
    AToReal,
    IToA,
    HexToA,
    OctToA,
    BinToA,
    RealToA,
    RandMode,
    ConstraintMode,
};

SLANG_EXPORT std::ostream& operator<<(std::ostream& os, KnownSystemName ksn);
SLANG_EXPORT std::string_view toString(KnownSystemName ksn);
SLANG_EXPORT KnownSystemName parseKnownSystemName(std::string_view str);

class SLANG_EXPORT KnownSystemName_traits {
public:
    static const std::array<KnownSystemName, 286> values;
};

}
